// ColorSensor Library for Studuino
//
#include "ColorSensor.h"
#include <Wire.h>
#include <Arduino.h>

COLOR::COLOR() {
}

bool COLOR::testConnection() {
  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_ID);
  Wire.endTransmission();  

  Wire.requestFrom(COLOR_SENSOR_ADDR, 1);
  if(Wire.available()) {
    byte msg = Wire.read();
    // ID register contains PARTNO(upper 4bits) and REVNO(lower 4bits).
    // TCS3410 and TCS341x's Part No. are 0000 and 0001, respectively.
    return ((msg & 0xf0) >> 4 == 0x01);   
  }
  return false;
}

void COLOR::initialize() {
  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_TIMING);
  Wire.write(INTEG_MODE_FREE);
  Wire.endTransmission();  

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_INT_SOURCE);
  Wire.write(INT_SOURCE_GREEN);
  Wire.endTransmission();  

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_INT);
  Wire.write(INTR_LEVEL|INTR_PERSIST_EVERY);
  Wire.endTransmission();  

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_GAIN);
  Wire.write(GAIN_1|PRESCALER_4);
  Wire.endTransmission();
}

void COLOR::update() {
  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_CTL);
  Wire.write(CTL_DAT_INIITIATE);
  Wire.endTransmission();  

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(REG_BLOCK_READ);
  Wire.endTransmission();

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.requestFrom(COLOR_SENSOR_ADDR,8);
  if(8<= Wire.available())  { 
    for(byte i = 0; i < 8; i++)
    {
      readingdata[i]=Wire.read();
      //Serial.println(readingdata[i],BIN);
    }
  }
  green = readingdata[1] * 256 + readingdata[0];
  red = readingdata[3] * 256 + readingdata[2];
  blue = readingdata[5] * 256 + readingdata[4];
  clr = readingdata[7] * 256 + readingdata[6];

  X = (-0.14282) * red + (1.54924) * green + (-0.95641) * blue;
  Y = (-0.32466) * red + (1.57837) * green + (-0.73191) * blue;
  Z = (-0.68202) * red + (0.77073) * green + (0.56332) * blue;
  x = X / (X + Y + Z);
  y = Y / (X + Y + Z);

  Wire.beginTransmission(COLOR_SENSOR_ADDR);
  Wire.write(CLR_INT);
  Wire.endTransmission();
}

// Getting the color code corresponding to Studuino ICON Programming.
int COLOR::getColorCode() {
  byte val = COLOR_UNDEF;

  if ( ( x > 0.5 ) && ( x < 0.6 ) && ( y > 0.44 ) && ( y < 0.46 ) ){  // Red
    val = COLOR_RED;
  }
  if ( ( x > 0.33 ) && ( x <= 0.38 ) && ( y >= 0.36 ) && ( y <= 0.39 ) && 
    (clr < 10000) && (green > 1500) && (red < 1000) && (blue < 1200)){  // Green
    val = COLOR_GREEN;
  }
  if ( ( x > 0.11 ) && ( x < 0.25 ) && ( y > 0.24 ) && ( y < 0.32 ) ){  // Blue
    val = COLOR_BLUE;
  }
  if ( ( x >= 0.34 ) && ( x <= 0.37 ) && ( y >= 0.36 ) && ( y <= 0.38 ) && (clr > 7000)){  // White
    val = COLOR_WHITE;
  }
  if ( ( x >= 0.41 ) && ( x <= 0.45 ) && ( y >= 0.41 ) && ( y <= 0.45 ) && (clr > 5000)){  // Yellow
    val = COLOR_YELLOW;
  }
  if ( ( x >= 0.42 ) && ( x <= 0.47 ) && ( y >= 0.41 ) && ( y <= 0.43 ) && (clr < 5000)){  // Brown
    val = COLOR_BROWN;
  }
  if ((red < 1000) && (green < 1000) && (blue < 1000)){  // Black
    val = COLOR_BLACK;
  }

  return val;
}

// Getting sensor value of each filter;
unsigned int COLOR::getColorRed() {
  return red;
}
unsigned int COLOR::getColorGreen() {
  return green;
}
unsigned int COLOR::getColorBlue() {
  return blue;
}
unsigned int COLOR::getColorClear() {
  return clr;
}

// Getting x value of chromaticity coordinate(x, y).
double COLOR::getX() {
  return x;
}

// Getting y value of chromaticity coordinate(x, y).
double COLOR::getY() {
  return y;
}

